@@ -1,4 +1,4 @@
-#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -26,11 +26,10 @@ branches and make a code block out of it. But it doesn't contain any target
 language syntax.
 """
 
-from nuitka import Options
 from nuitka.__past__ import iterItems
 from nuitka.PythonVersions import python_version
 
-from . import Contexts, Emission, Helpers
+from . import Contexts, Emission
 from .AttributeCodes import (
     generateAssignmentAttributeCode,
     generateAttributeLookupCode,
@@ -55,9 +54,13 @@ from .BuiltinCodes import (
     generateBuiltinRange2Code,
     generateBuiltinRange3Code,
     generateBuiltinRefCode,
+    generateBuiltinSum1Code,
+    generateBuiltinSum2Code,
     generateBuiltinType1Code,
     generateBuiltinType3Code,
-    generateBuiltinXrangeCode
+    generateBuiltinXrange1Code,
+    generateBuiltinXrange2Code,
+    generateBuiltinXrange3Code
 )
 from .CallCodes import generateCallCode, getCallsCode, getCallsDecls
 from .ClassCodes import (
@@ -70,7 +73,13 @@ from .ConditionalCodes import (
     generateConditionalAndOrCode,
     generateConditionalCode
 )
-from .ConstantCodes import generateConstantReferenceCode
+from .ConstantCodes import (
+    generateConstantEllipsisReferenceCode,
+    generateConstantFalseReferenceCode,
+    generateConstantNoneReferenceCode,
+    generateConstantReferenceCode,
+    generateConstantTrueReferenceCode
+)
 from .CoroutineCodes import (
     generateAsyncIterCode,
     generateAsyncNextCode,
@@ -82,6 +91,7 @@ from .DictCodes import (
     generateBuiltinDictCode,
     generateDictionaryCreationCode,
     generateDictOperationGetCode,
+    generateDictOperationInCode,
     generateDictOperationRemoveCode,
     generateDictOperationSetCode,
     generateDictOperationUpdateCode
@@ -101,11 +111,13 @@ from .ExceptionCodes import (
     generateExceptionPublishCode,
     generateExceptionRefCode
 )
-from .ExpressionCodes import generateExpressionOnlyCode, generateSideEffectsCode
+from .ExpressionCodes import (
+    generateExpressionOnlyCode,
+    generateSideEffectsCode
+)
 from .FrameCodes import (
     generateFramePreserveExceptionCode,
-    generateFrameRestoreExceptionCode,
-    generateStatementsFrameCode
+    generateFrameRestoreExceptionCode
 )
 from .FunctionCodes import (
     generateFunctionCallCode,
@@ -117,7 +129,6 @@ from .FunctionCodes import (
     getFunctionDirectDecl
 )
 from .GeneratorCodes import (
-    generateGeneratorEntryCode,
     generateMakeGeneratorObjectCode,
     getGeneratorObjectCode
 )
@@ -128,7 +139,11 @@ from .GlobalsLocalsCodes import (
     generateBuiltinVarsCode,
     generateSetLocalsCode
 )
-from .Helpers import generateStatementCode
+from .Helpers import (
+    generateStatementSequenceCode,
+    setExpressionDispatchDict,
+    setStatementDispatchDict
+)
 from .IdCodes import generateBuiltinHashCode, generateBuiltinIdCode
 from .ImportCodes import (
     generateBuiltinImportCode,
@@ -147,7 +162,6 @@ from .IteratorCodes import (
     generateSpecialUnpackCode,
     generateUnpackCheckCode
 )
-from .LabelCodes import getStatementTrace
 from .ListCodes import (
     generateBuiltinListCode,
     generateListCreationCode,
@@ -180,6 +194,7 @@ from .ReturnCodes import (
 from .SetCodes import (
     generateBuiltinSetCode,
     generateSetCreationCode,
+    generateSetLiteralCreationCode,
     generateSetOperationAddCode,
     generateSetOperationUpdateCode
 )
@@ -190,10 +205,13 @@ from .SliceCodes import (
     generateSliceLookupCode
 )
 from .StringCodes import (
+    generateBuiltinAsciiCode,
     generateBuiltinChrCode,
+    generateBuiltinFormatCode,
     generateBuiltinOrdCode,
     generateBuiltinStrCode,
-    generateBuiltinUnicodeCode
+    generateBuiltinUnicodeCode,
+    generateStringContenationCode
 )
 from .SubscriptCodes import (
     generateAssignmentSubscriptCode,
@@ -311,66 +329,9 @@ def generateFunctionBodyCode(function_body, context):
             )
         )
 
-    return function_code
+    return function_code, function_context
 
 
-def _generateStatementSequenceCode(statement_sequence, emit, context):
-    if statement_sequence is None:
-        return
-
-    for statement in statement_sequence.getStatements():
-        if Options.shallTraceExecution():
-            source_ref = statement.getSourceReference()
-
-            statement_repr = repr(statement)
-            source_repr = source_ref.getAsString()
-
-            if python_version >= 300:
-                statement_repr = statement_repr.encode("utf8")
-                source_repr = source_repr.encode("utf8")
-
-            emit(
-                getStatementTrace(
-                    source_repr,
-                    statement_repr
-                )
-            )
-
-        # Might contain frame statement sequences as children.
-        if statement.isStatementsFrame():
-            generateStatementsFrameCode(
-                statement_sequence = statement,
-                emit               = emit,
-                context            = context
-            )
-        else:
-            generateStatementCode(
-                statement = statement,
-                emit      = emit,
-                context   = context
-            )
-
-
-def generateStatementSequenceCode(statement_sequence, emit, context,
-                                  allow_none = False):
-
-    if allow_none and statement_sequence is None:
-        return None
-
-    assert statement_sequence.kind == "STATEMENTS_SEQUENCE", statement_sequence
-
-    statement_context = Contexts.PythonStatementCContext(context)
-
-    _generateStatementSequenceCode(
-        statement_sequence = statement_sequence,
-        emit               = emit,
-        context            = statement_context
-    )
-
-    # Complain if any temporary was not dealt with yet.
-    assert not statement_context.getCleanupTempnames(), \
-      statement_context.getCleanupTempnames()
-
 
 def prepareModuleCode(global_context, module, module_name):
     # As this not only creates all modules, but also functions, it deals
@@ -403,23 +364,27 @@ def prepareModuleCode(global_context, module, module_name):
     function_body_codes = []
 
     for function_body in module.getUsedFunctions():
-        function_code = generateFunctionBodyCode(
+        function_code, function_context = generateFunctionBodyCode(
             function_body = function_body,
             context       = context
         )
 
-        assert type(function_code) is str
+        assert type(function_code) is str, type(function_code)
 
         function_body_codes.append(function_code)
 
         function_decl = generateFunctionDeclCode(
-            function_body = function_body
+            function_body = function_body,
+            context       = function_context
+
         )
 
         if function_decl is not None:
             function_decl_codes.append(function_decl)
 
 
+    # These are for functions used from other modules. Due to cyclic
+    # dependencies, we cannot rely on those to be already created.
     for function_body in module.getCrossUsedFunctions():
         assert function_body.isCrossModuleUsed()
 
@@ -428,6 +393,10 @@ def prepareModuleCode(global_context, module, module_name):
             closure_variables   = function_body.getClosureVariables(),
             file_scope          = getExportScopeCode(
                 cross_module = function_body.isCrossModuleUsed()
+            ),
+            context             = Contexts.PythonFunctionDirectContext(
+                parent   = context,
+                function = function_body
             )
         )
 
@@ -481,7 +450,7 @@ def makeGlobalContext():
     return Contexts.PythonGlobalContext()
 
 
-Helpers.setExpressionDispatchDict(
+setExpressionDispatchDict(
     {
         "EXPRESSION_ATTRIBUTE_LOOKUP"               : generateAttributeLookupCode,
         "EXPRESSION_ATTRIBUTE_LOOKUP_SPECIAL"       : generateAttributeLookupSpecialCode,
@@ -496,6 +465,8 @@ Helpers.setExpressionDispatchDict(
         "EXPRESSION_BUILTIN_ITER2"                  : generateBuiltinIter2Code,
         "EXPRESSION_BUILTIN_NEXT1"                  : generateBuiltinNext1Code,
         "EXPRESSION_BUILTIN_NEXT2"                  : generateBuiltinNext2Code,
+        "EXPRESSION_BUILTIN_SUM1"                   : generateBuiltinSum1Code,
+        "EXPRESSION_BUILTIN_SUM2"                   : generateBuiltinSum2Code,
         "EXPRESSION_BUILTIN_TYPE1"                  : generateBuiltinType1Code,
         "EXPRESSION_BUILTIN_TYPE3"                  : generateBuiltinType3Code,
         "EXPRESSION_BUILTIN_IMPORT"                 : generateBuiltinImportCode,
@@ -530,7 +501,9 @@ Helpers.setExpressionDispatchDict(
         "EXPRESSION_BUILTIN_RANGE1"                 : generateBuiltinRange1Code,
         "EXPRESSION_BUILTIN_RANGE2"                 : generateBuiltinRange2Code,
         "EXPRESSION_BUILTIN_RANGE3"                 : generateBuiltinRange3Code,
-        "EXPRESSION_BUILTIN_XRANGE"                 : generateBuiltinXrangeCode,
+        "EXPRESSION_BUILTIN_XRANGE1"                : generateBuiltinXrange1Code,
+        "EXPRESSION_BUILTIN_XRANGE2"                : generateBuiltinXrange2Code,
+        "EXPRESSION_BUILTIN_XRANGE3"                : generateBuiltinXrange3Code,
         "EXPRESSION_BUILTIN_MAKE_EXCEPTION"         : generateBuiltinMakeExceptionCode,
         "EXPRESSION_BUILTIN_REF"                    : generateBuiltinRefCode,
         "EXPRESSION_BUILTIN_EXCEPTION_REF"          : generateExceptionRefCode,
@@ -542,33 +515,55 @@ Helpers.setExpressionDispatchDict(
         "EXPRESSION_CALL_KEYWORDS_ONLY"             : generateCallCode,
         "EXPRESSION_CALL_NO_KEYWORDS"               : generateCallCode,
         "EXPRESSION_CALL"                           : generateCallCode,
-        "EXPRESSION_CONSTANT_REF"                   : generateConstantReferenceCode,
+        "EXPRESSION_CONSTANT_NONE_REF"              : generateConstantNoneReferenceCode,
+        "EXPRESSION_CONSTANT_TRUE_REF"              : generateConstantTrueReferenceCode,
+        "EXPRESSION_CONSTANT_FALSE_REF"             : generateConstantFalseReferenceCode,
+        "EXPRESSION_CONSTANT_STR_REF"               : generateConstantReferenceCode,
+        "EXPRESSION_CONSTANT_UNICODE_REF"           : generateConstantReferenceCode,
+        "EXPRESSION_CONSTANT_BYTES_REF"             : generateConstantReferenceCode,
+        "EXPRESSION_CONSTANT_INT_REF"               : generateConstantReferenceCode,
+        "EXPRESSION_CONSTANT_LONG_REF"              : generateConstantReferenceCode,
+        "EXPRESSION_CONSTANT_FLOAT_REF"             : generateConstantReferenceCode,
+        "EXPRESSION_CONSTANT_COMPLEX_REF"           : generateConstantReferenceCode,
+        "EXPRESSION_CONSTANT_ELLIPSIS_REF"          : generateConstantEllipsisReferenceCode,
+        "EXPRESSION_CONSTANT_DICT_REF"              : generateConstantReferenceCode,
+        "EXPRESSION_CONSTANT_DICT_EMPTY_REF"        : generateConstantReferenceCode,
+        "EXPRESSION_CONSTANT_TUPLE_REF"             : generateConstantReferenceCode,
+        "EXPRESSION_CONSTANT_LIST_REF"              : generateConstantReferenceCode,
+        "EXPRESSION_CONSTANT_SET_REF"               : generateConstantReferenceCode,
+        "EXPRESSION_CONSTANT_SLICE_REF"             : generateConstantReferenceCode,
+        "EXPRESSION_CONSTANT_XRANGE_REF"            : generateConstantReferenceCode,
+        "EXPRESSION_CONSTANT_TYPE_REF"              : generateConstantReferenceCode,
         "EXPRESSION_CONDITIONAL"                    : generateConditionalCode,
         "EXPRESSION_CONDITIONAL_OR"                 : generateConditionalAndOrCode,
         "EXPRESSION_CONDITIONAL_AND"                : generateConditionalAndOrCode,
         "EXPRESSION_COMPARISON"                     : generateComparisonExpressionCode,
         "EXPRESSION_COMPARISON_IS"                  : generateComparisonExpressionCode,
         "EXPRESSION_COMPARISON_IS_NOT"              : generateComparisonExpressionCode,
+        "EXPRESSION_COMPARISON_IN"                  : generateComparisonExpressionCode,
+        "EXPRESSION_COMPARISON_NOT_IN"              : generateComparisonExpressionCode,
         "EXPRESSION_COMPARISON_EXCEPTION_MATCH"     : generateComparisonExpressionCode,
-        "EXPRESSION_DICT_OPERATION_UPDATE"          : generateDictOperationUpdateCode,
         "EXPRESSION_DICT_OPERATION_GET"             : generateDictOperationGetCode,
-        "EXPRESSION_DICT_OPERATION_SET"             : generateDictOperationSetCode,
+        "EXPRESSION_DICT_OPERATION_IN"              : generateDictOperationInCode,
+        "EXPRESSION_DICT_OPERATION_NOT_IN"          : generateDictOperationInCode,
         "EXPRESSION_FUNCTION_CREATION"              : generateFunctionCreationCode,
         "EXPRESSION_FUNCTION_CALL"                  : generateFunctionCallCode,
         "EXPRESSION_IMPORT_MODULE"                  : generateImportModuleCode,
         "EXPRESSION_IMPORT_MODULE_HARD"             : generateImportModuleHardCode,
         "EXPRESSION_IMPORT_NAME"                    : generateImportNameCode,
-        "EXPRESSION_LIST_OPERATION_APPEND"          : generateListOperationAppendCode,
         "EXPRESSION_LIST_OPERATION_EXTEND"          : generateListOperationExtendCode,
         "EXPRESSION_LIST_OPERATION_POP"             : generateListOperationPopCode,
         "EXPRESSION_MODULE_FILE_ATTRIBUTE_REF"      : generateModuleFileAttributeCode,
         "EXPRESSION_MAKE_GENERATOR_OBJECT"          : generateMakeGeneratorObjectCode,
         "EXPRESSION_MAKE_COROUTINE_OBJECT"          : generateMakeCoroutineObjectCode,
         "EXPRESSION_MAKE_SET"                       : generateSetCreationCode,
+        "EXPRESSION_MAKE_SET_LITERAL"               : generateSetLiteralCreationCode,
         "EXPRESSION_MAKE_TUPLE"                     : generateTupleCreationCode,
         "EXPRESSION_MAKE_LIST"                      : generateListCreationCode,
         "EXPRESSION_MAKE_DICT"                      : generateDictionaryCreationCode,
         "EXPRESSION_OPERATION_BINARY"               : generateOperationBinaryCode,
+        "EXPRESSION_OPERATION_BINARY_ADD"           : generateOperationBinaryCode,
+        "EXPRESSION_OPERATION_BINARY_MULT"          : generateOperationBinaryCode,
         "EXPRESSION_OPERATION_BINARY_INPLACE"       : generateOperationBinaryCode,
         "EXPRESSION_OPERATION_UNARY"                : generateOperationUnaryCode,
         "EXPRESSION_OPERATION_NOT"                  : generateOperationUnaryCode,
@@ -576,7 +571,6 @@ Helpers.setExpressionDispatchDict(
         "EXPRESSION_RETURNED_VALUE_REF"             : generateReturnedValueRefCode,
         "EXPRESSION_SUBSCRIPT_LOOKUP"               : generateSubscriptLookupCode,
         "EXPRESSION_SLICE_LOOKUP"                   : generateSliceLookupCode,
-        "EXPRESSION_SET_OPERATION_ADD"              : generateSetOperationAddCode,
         "EXPRESSION_SET_OPERATION_UPDATE"           : generateSetOperationUpdateCode,
         "EXPRESSION_SIDE_EFFECTS"                   : generateSideEffectsCode,
         "EXPRESSION_SPECIAL_UNPACK"                 : generateSpecialUnpackCode,
@@ -588,10 +582,13 @@ Helpers.setExpressionDispatchDict(
         "EXPRESSION_ASYNC_WAIT"                     : generateAsyncWaitCode,
         "EXPRESSION_ASYNC_ITER"                     : generateAsyncIterCode,
         "EXPRESSION_ASYNC_NEXT"                     : generateAsyncNextCode,
+        "EXPRESSION_STRING_CONCATENATION"           : generateStringContenationCode,
+        "EXPRESSION_BUILTIN_FORMAT"                 : generateBuiltinFormatCode,
+        "EXPRESSION_BUILTIN_ASCII"                  : generateBuiltinAsciiCode,
     }
 )
 
-Helpers.setStatementDispatchDict(
+setStatementDispatchDict(
     {
         "STATEMENT_ASSIGNMENT_VARIABLE"      : generateAssignmentVariableCode,
         "STATEMENT_ASSIGNMENT_ATTRIBUTE"     : generateAssignmentAttributeCode,
@@ -602,6 +599,7 @@ Helpers.setStatementDispatchDict(
         "STATEMENT_DEL_SUBSCRIPT"            : generateDelSubscriptCode,
         "STATEMENT_DEL_SLICE"                : generateDelSliceCode,
         "STATEMENT_DICT_OPERATION_REMOVE"    : generateDictOperationRemoveCode,
+        "STATEMENT_DICT_OPERATION_UPDATE"    : generateDictOperationUpdateCode,
         "STATEMENT_RELEASE_VARIABLE"         : generateVariableReleaseCode,
         "STATEMENT_EXPRESSION_ONLY"          : generateExpressionOnlyCode,
         "STATEMENT_RETURN"                   : generateReturnCode,
@@ -611,6 +609,9 @@ Helpers.setStatementDispatchDict(
         "STATEMENT_PRINT_VALUE"              : generatePrintValueCode,
         "STATEMENT_PRINT_NEWLINE"            : generatePrintNewlineCode,
         "STATEMENT_IMPORT_STAR"              : generateImportStarCode,
+        "STATEMENT_LIST_OPERATION_APPEND"    : generateListOperationAppendCode,
+        "STATEMENT_SET_OPERATION_ADD"        : generateSetOperationAddCode,
+        "STATEMENT_DICT_OPERATION_SET"       : generateDictOperationSetCode,
         "STATEMENT_LOOP"                     : generateLoopCode,
         "STATEMENT_LOOP_BREAK"               : generateLoopBreakCode,
         "STATEMENT_LOOP_CONTINUE"            : generateLoopContinueCode,
@@ -620,7 +621,6 @@ Helpers.setStatementDispatchDict(
         "STATEMENT_EXEC"                     : generateExecCode,
         "STATEMENT_LOCALS_DICT_SYNC"         : generateLocalsDictSyncCode,
         "STATEMENT_SET_LOCALS"               : generateSetLocalsCode,
-        "STATEMENT_GENERATOR_ENTRY"          : generateGeneratorEntryCode,
         "STATEMENT_PRESERVE_FRAME_EXCEPTION" : generateFramePreserveExceptionCode,
         "STATEMENT_RESTORE_FRAME_EXCEPTION"  : generateFrameRestoreExceptionCode,
         "STATEMENT_PUBLISH_EXCEPTION"        : generateExceptionPublishCode
