@@ -1,4 +1,4 @@
-#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -22,8 +22,8 @@
 from nuitka.PythonVersions import python_version
 
 from .Helpers import generateExpressionCode
-from .templates. \
-    CodeTemplatesExceptions import template_publish_exception_to_handler
+from .templates.CodeTemplatesExceptions import \
+    template_publish_exception_to_handler
 
 
 def getExceptionIdentifier(exception_type):
