@@ -1,4 +1,4 @@
-#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -77,19 +77,31 @@ template_frame_guard_full_exception_handler = """\
 RESTORE_FRAME_EXCEPTION( %(frame_identifier)s );
 #endif
 
-if ( exception_tb == NULL )
 {
-    exception_tb = %(tb_making)s;
-}
-else if ( exception_tb->tb_frame != %(frame_identifier)s )
-{
-    PyTracebackObject *traceback_new = MAKE_TRACEBACK( %(frame_identifier)s );
-    traceback_new->tb_next = exception_tb;
-    exception_tb = traceback_new;
-}
+    bool needs_detach = false;
 
+    if ( exception_tb == NULL )
+    {
+        exception_tb = %(tb_making)s;
+        needs_detach = true;
+    }
+    else if ( exception_lineno != -1 )
+    {
+        PyTracebackObject *traceback_new = MAKE_TRACEBACK( %(frame_identifier)s, exception_lineno );
+        traceback_new->tb_next = exception_tb;
+        exception_tb = traceback_new;
+
+        needs_detach = true;
+    }
+
+    if (needs_detach)
+    {
 %(store_frame_locals)s
 
+        detachFrame( exception_tb, %(frame_locals_name)s );
+    }
+}
+
 popFrameStack();
 
 #if PYTHON_VERSION >= 340
@@ -101,16 +113,11 @@ Py_DECREF( %(frame_identifier)s );
 goto %(parent_exception_exit)s;
 """
 
-template_frame_guard_once_decl = """\
-static PyFrameObject *cache_%(frame_identifier)s = NULL;
-"""
-
-
 # Frame for a module. TODO: Use it for functions called only once.
 # TODO: The once guard need not take a reference count in its frame class.
 template_frame_guard_once = """\
 // Frame without reuse.
-%(frame_identifier)s = MAKE_FRAME( %(code_identifier)s, %(module_identifier)s );
+%(frame_identifier)s = MAKE_MODULE_FRAME( %(code_identifier)s, %(module_identifier)s );
 
 // Push the new frame as the currently active one, and we should be exclusively
 // owning it.
@@ -145,7 +152,7 @@ if ( exception_tb == NULL )
 }
 else if ( exception_tb->tb_frame != %(frame_identifier)s )
 {
-    PyTracebackObject *traceback_new = MAKE_TRACEBACK( %(frame_identifier)s );
+    PyTracebackObject *traceback_new = MAKE_TRACEBACK( %(frame_identifier)s, exception_lineno );
     traceback_new->tb_next = exception_tb;
     exception_tb = traceback_new;
 }
@@ -162,43 +169,14 @@ Py_DECREF( %(frame_identifier)s );
 goto %(parent_exception_exit)s;
 %(no_exception_exit)s:;"""
 
-template_generator_initial_throw = """\
-// Throwing into unstarted generators is possible. As they don't stand any
-// chance to deal with them, we might as well create traceback on the
-// outside,
-if ( generator->m_exception_type )
-{
-    generator->m_yielded = NULL;
-
-    exception_type = generator->m_exception_type;
-    generator->m_exception_type = NULL;
-
-    exception_value = generator->m_exception_value;
-    generator->m_exception_value = NULL;
-
-    exception_tb = generator->m_exception_tb;;
-    generator->m_exception_tb = NULL;
-
-    if (exception_tb == NULL)
-    {
-        goto %(frame_exception_exit)s;
-    }
-    else
-    {
-        goto function_exception_exit;
-    }
-}
-"""
-
 # Frame in a generator
 template_frame_guard_generator = """\
-MAKE_OR_REUSE_FRAME( cache_%(frame_identifier)s, %(code_identifier)s, %(module_identifier)s );
-%(frame_identifier)s = cache_%(frame_identifier)s;
+MAKE_OR_REUSE_FRAME( %(frame_cache_identifier)s, %(code_identifier)s, %(module_identifier)s );
+generator->m_frame = %(frame_cache_identifier)s;
+Py_INCREF( generator->m_frame );
 
-Py_INCREF( %(frame_identifier)s );
-generator->m_frame = %(frame_identifier)s;
 #if PYTHON_VERSION >= 340
-%(frame_identifier)s->f_gen = (PyObject *)generator;
+generator->m_frame->f_gen = (PyObject *)generator;
 #endif
 
 Py_CLEAR( generator->m_frame->f_back );
@@ -210,59 +188,141 @@ PyThreadState_GET()->frame = generator->m_frame;
 Py_INCREF( generator->m_frame );
 
 #if PYTHON_VERSION >= 340
-%(frame_identifier)s->f_executing += 1;
+generator->m_frame->f_executing += 1;
+#endif
+
+#if PYTHON_VERSION >= 300
+// Accept currently existing exception as the one to publish again when we
+// yield or yield from.
+
+PyThreadState *thread_state = PyThreadState_GET();
+
+generator->m_frame->f_exc_type = thread_state->exc_type;
+if ( generator->m_frame->f_exc_type == Py_None ) generator->m_frame->f_exc_type = NULL;
+Py_XINCREF( generator->m_frame->f_exc_type );
+generator->m_frame->f_exc_value = thread_state->exc_value;
+Py_XINCREF( generator->m_frame->f_exc_value );
+generator->m_frame->f_exc_traceback = thread_state->exc_traceback;
+Py_XINCREF( generator->m_frame->f_exc_traceback );
 #endif
 
 // Framed code:
 %(codes)s
 
 #if PYTHON_VERSION >= 340
-%(frame_identifier)s->f_executing -= 1;
+generator->m_frame->f_executing -= 1;
 #endif
 
-Py_DECREF( %(frame_identifier)s );
+#if PYTHON_VERSION >= 300
+Py_CLEAR( generator->m_frame->f_exc_type );
+Py_CLEAR( generator->m_frame->f_exc_value );
+Py_CLEAR( generator->m_frame->f_exc_traceback );
+#endif
+
+Py_DECREF( generator->m_frame );
 goto %(no_exception_exit)s;
 """
 
+# Frame in a coroutine
+template_frame_guard_coroutine = """\
+MAKE_OR_REUSE_FRAME( %(frame_cache_identifier)s, %(code_identifier)s, %(module_identifier)s );
+coroutine->m_frame = %(frame_cache_identifier)s;
+Py_INCREF( coroutine->m_frame );
+
+coroutine->m_frame->f_gen = (PyObject *)coroutine;
+
+Py_CLEAR( coroutine->m_frame->f_back );
+
+coroutine->m_frame->f_back = PyThreadState_GET()->frame;
+Py_INCREF( coroutine->m_frame->f_back );
+
+PyThreadState_GET()->frame = coroutine->m_frame;
+Py_INCREF( coroutine->m_frame );
+
+coroutine->m_frame->f_executing += 1;
+
+// Accept currently existing exception as the one to publish again when we
+// yield or yield from.
+
+PyThreadState *thread_state = PyThreadState_GET();
+
+coroutine->m_frame->f_exc_type = thread_state->exc_type;
+if ( coroutine->m_frame->f_exc_type == Py_None ) coroutine->m_frame->f_exc_type = NULL;
+Py_XINCREF( coroutine->m_frame->f_exc_type );
+coroutine->m_frame->f_exc_value = thread_state->exc_value;
+Py_XINCREF( coroutine->m_frame->f_exc_value );
+coroutine->m_frame->f_exc_traceback = thread_state->exc_traceback;
+Py_XINCREF( coroutine->m_frame->f_exc_traceback );
+
+// Framed code:
+%(codes)s
+
+coroutine->m_frame->f_executing -= 1;
+
+Py_CLEAR( coroutine->m_frame->f_exc_type );
+Py_CLEAR( coroutine->m_frame->f_exc_value );
+Py_CLEAR( coroutine->m_frame->f_exc_traceback );
+
+Py_DECREF( coroutine->m_frame );
+goto %(no_exception_exit)s;
+"""
+
+
+# TODO: This cannot happen, can it?
 template_frame_guard_generator_return_handler = """\
 %(frame_return_exit)s:;
-#if PYTHON_VERSION > 300
-RESTORE_FRAME_EXCEPTION( %(frame_identifier)s );
+
+#if PYTHON_VERSION >= 300
+Py_CLEAR( %(frame_identifier)s->f_exc_type );
+Py_CLEAR( %(frame_identifier)s->f_exc_value );
+Py_CLEAR( %(frame_identifier)s->f_exc_traceback );
 #endif
+
 Py_DECREF( %(frame_identifier)s );
 goto %(return_exit)s;
 """
 
 
-# TODO: Should we check type of value.
 template_frame_guard_generator_exception_handler = """\
 %(frame_exception_exit)s:;
 
+// If it's not an exit exception, consider and create a traceback for it.
 if ( !EXCEPTION_MATCH_GENERATOR( exception_type ) )
 {
+    int needs_detach = false;
+
     if ( exception_tb == NULL )
     {
         exception_tb = %(tb_making)s;
+        needs_detach = true;
     }
     else if ( exception_tb->tb_frame != %(frame_identifier)s )
     {
-        PyTracebackObject *traceback_new = MAKE_TRACEBACK( %(frame_identifier)s );
+        PyTracebackObject *traceback_new = MAKE_TRACEBACK( %(frame_identifier)s, exception_lineno );
         traceback_new->tb_next = exception_tb;
         exception_tb = traceback_new;
+
+        needs_detach = true;
     }
 
+    if (needs_detach)
+    {
 %(store_frame_locals)s
+
+        detachFrame( exception_tb, %(frame_locals_name)s );
+    }
 }
 
-#if PYTHON_VERSION > 300
-RESTORE_FRAME_EXCEPTION( %(frame_identifier)s );
+#if PYTHON_VERSION >= 300
+Py_CLEAR( %(frame_identifier)s->f_exc_type );
+Py_CLEAR( %(frame_identifier)s->f_exc_value );
+Py_CLEAR( %(frame_identifier)s->f_exc_traceback );
 #endif
 
 Py_DECREF( %(frame_identifier)s );
 // Return the error.
 goto %(parent_exception_exit)s;
-%(no_exception_exit)s:;
 """
 
-template_frame_locals_update = """\
-detachFrame( exception_tb, %(locals_identifier)s );"""
+from . import TemplateDebugWrapper # isort:skip
+TemplateDebugWrapper.checkDebug(globals())
