@@ -1,4 +1,4 @@
-#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -23,7 +23,9 @@ code.
 
 import sys
 
-from PyQt4 import QtCore, QtGui, uic  # pylint: disable=F0401,I0021
+from PyQt5 import QtCore  # @UnresolvedImport pylint: disable=F0401,I0021
+from PyQt5 import QtGui  # @UnresolvedImport pylint: disable=F0401,I0021
+from PyQt5 import uic  # @UnresolvedImport pylint: disable=F0401,I0021
 
 from nuitka import SourceCodeReferences
 from nuitka.utils import Utils
