@@ -1,4 +1,4 @@
-#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -25,15 +25,20 @@ and then later active again, via another import, and in this case, we should
 not start anew, but reuse what we already found out about it.
 """
 
-from nuitka.plugins.PluginBase import Plugins
+from nuitka.plugins.Plugins import Plugins
 from nuitka.utils import Utils
 
 imported_modules = {}
 imported_by_name = {}
 
 def addImportedModule(imported_module):
+    module_filename = Utils.relpath(imported_module.getFilename())
+
+    if Utils.basename(module_filename) == "__init__.py":
+        module_filename = Utils.dirname(module_filename)
+
     key = (
-        Utils.relpath(imported_module.getFilename()),
+        module_filename,
         imported_module.getFullName()
     )
 
@@ -71,3 +76,19 @@ def getImportedModuleByPath(module_relpath):
             return imported_modules[key]
 
     raise KeyError(module_relpath)
+
+
+def replaceImportedModule(old, new):
+    for key, value in imported_by_name.items():
+        if value == old:
+            imported_by_name[key] = new
+            break
+    else:
+        assert False
+
+    for key, value in imported_modules.items():
+        if value == old:
+            imported_modules[key] = new
+            break
+    else:
+        assert False
