@@ -1,4 +1,4 @@
-#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -19,6 +19,10 @@
 
 """
 
+import functools
+
+from nuitka.PythonVersions import needsSetLiteralReverseInsertion
+
 from .NodeBases import (
     ExpressionChildrenHavingBase,
     SideEffectsFromChildrenMixin
@@ -28,6 +32,11 @@ from .NodeMakingHelpers import (
     makeStatementOnlyNodesFromExpressions,
     wrapExpressionWithSideEffects
 )
+from .shapes.BuiltinTypeShapes import (
+    ShapeTypeList,
+    ShapeTypeSet,
+    ShapeTypeTuple
+)
 
 
 class ExpressionMakeSequenceBase(SideEffectsFromChildrenMixin,
@@ -65,7 +74,7 @@ class ExpressionMakeSequenceBase(SideEffectsFromChildrenMixin,
         # Abstract method, pylint: disable=R0201
         return None
 
-    def computeExpression(self, constraint_collection):
+    def computeExpression(self, trace_collection):
         elements = self.getElements()
 
         for count, element in enumerate(elements):
@@ -103,11 +112,14 @@ class ExpressionMakeSequenceBase(SideEffectsFromChildrenMixin,
     def isKnownToBeIterable(self, count):
         return count is None or count == len(self.getElements())
 
+    def isKnownToBeIterableAtMin(self, count):
+        return count <= len(self.getElements())
+
     def getIterationValue(self, count):
         return self.getElements()[count]
 
-    def getIterationLength(self):
-        return len(self.getElements())
+    def getIterationValueRange(self, start, stop):
+        return self.getElements()[start:stop]
 
     @staticmethod
     def canPredictIterationValues():
@@ -129,7 +141,7 @@ class ExpressionMakeSequenceBase(SideEffectsFromChildrenMixin,
     def mayBeNone(self):
         return False
 
-    def computeExpressionDrop(self, statement, constraint_collection):
+    def computeExpressionDrop(self, statement, trace_collection):
         result = makeStatementOnlyNodesFromExpressions(
             expressions = self.getElements()
         )
@@ -149,9 +161,15 @@ class ExpressionMakeTuple(ExpressionMakeSequenceBase):
             source_ref    = source_ref
         )
 
+    def getTypeShape(self):
+        return ShapeTypeTuple
+
     def getSimulator(self):
         return tuple
 
+    def getIterationLength(self):
+        return len(self.getElements())
+
 
 class ExpressionMakeList(ExpressionMakeSequenceBase):
     kind = "EXPRESSION_MAKE_LIST"
@@ -164,10 +182,16 @@ class ExpressionMakeList(ExpressionMakeSequenceBase):
             source_ref    = source_ref
         )
 
+    def getTypeShape(self):
+        return ShapeTypeList
+
     def getSimulator(self):
         return list
 
-    def computeExpressionIter1(self, iter_node, constraint_collection):
+    def getIterationLength(self):
+        return len(self.getElements())
+
+    def computeExpressionIter1(self, iter_node, trace_collection):
         result = ExpressionMakeTuple(
             elements   = self.getElements(),
             source_ref = self.source_ref
@@ -190,9 +214,32 @@ class ExpressionMakeSet(ExpressionMakeSequenceBase):
             source_ref    = source_ref
         )
 
+    def getTypeShape(self):
+        return ShapeTypeSet
+
     def getSimulator(self):
         return set
 
+    def getIterationLength(self):
+        element_count = len(self.getElements())
+
+        # Hashing may consume elements.
+        if element_count >= 2:
+            return None
+        else:
+            return element_count
+
+    def getIterationMinLength(self):
+        element_count = len(self.getElements())
+
+        if element_count == 0:
+            return 0
+        else:
+            return 1
+
+    def getIterationMaxLength(self):
+        return len(self.getElements())
+
     def mayRaiseException(self, exception_type):
         for element in self.getElements():
             if not element.isKnownToBeHashable():
@@ -203,7 +250,7 @@ class ExpressionMakeSet(ExpressionMakeSequenceBase):
 
         return False
 
-    def computeExpressionIter1(self, iter_node, constraint_collection):
+    def computeExpressionIter1(self, iter_node, trace_collection):
         result = ExpressionMakeTuple(
             elements   = self.getElements(),
             source_ref = self.source_ref
@@ -213,3 +260,17 @@ class ExpressionMakeSet(ExpressionMakeSequenceBase):
 
         return iter_node, "new_expression", """\
 Iteration over set reduced to tuple."""
+
+
+class ExpressionMakeSetLiteral(ExpressionMakeSet):
+    kind = "EXPRESSION_MAKE_SET_LITERAL"
+
+    def getSimulator(self):
+        if needsSetLiteralReverseInsertion():
+            @functools.wraps(set)
+            def mySet(value):
+                return set(reversed(tuple(value)))
+
+            return mySet
+        else:
+            return set
