@@ -1,4 +1,4 @@
-#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -38,12 +38,13 @@ from nuitka.nodes.AttributeNodes import (
 )
 from nuitka.nodes.BuiltinDecodingNodes import (
     ExpressionBuiltinChr,
-    ExpressionBuiltinOrd,
-    ExpressionBuiltinOrd0
+    ExpressionBuiltinOrd
 )
 from nuitka.nodes.BuiltinDictNodes import ExpressionBuiltinDict
 from nuitka.nodes.BuiltinFormatNodes import (
+    ExpressionBuiltinAscii,
     ExpressionBuiltinBin,
+    ExpressionBuiltinFormat,
     ExpressionBuiltinHex,
     ExpressionBuiltinId,
     ExpressionBuiltinOct
@@ -51,23 +52,31 @@ from nuitka.nodes.BuiltinFormatNodes import (
 from nuitka.nodes.BuiltinHashNodes import ExpressionBuiltinHash
 from nuitka.nodes.BuiltinIteratorNodes import (
     ExpressionBuiltinIter1,
-    ExpressionBuiltinIter2,
-    ExpressionBuiltinLen,
+    ExpressionBuiltinIter2
+)
+from nuitka.nodes.BuiltinLenNodes import ExpressionBuiltinLen
+from nuitka.nodes.BuiltinNextNodes import (
     ExpressionBuiltinNext1,
     ExpressionBuiltinNext2
 )
 from nuitka.nodes.BuiltinOpenNodes import ExpressionBuiltinOpen
 from nuitka.nodes.BuiltinRangeNodes import (
-    ExpressionBuiltinRange0,
     ExpressionBuiltinRange1,
     ExpressionBuiltinRange2,
-    ExpressionBuiltinRange3
+    ExpressionBuiltinRange3,
+    ExpressionBuiltinXrange1,
+    ExpressionBuiltinXrange2,
+    ExpressionBuiltinXrange3
 )
 from nuitka.nodes.BuiltinRefNodes import (
     ExpressionBuiltinAnonymousRef,
     ExpressionBuiltinOriginalRef,
     ExpressionBuiltinRef
 )
+from nuitka.nodes.BuiltinSumNodes import (
+    ExpressionBuiltinSum1,
+    ExpressionBuiltinSum2
+)
 from nuitka.nodes.BuiltinTypeNodes import (
     ExpressionBuiltinBool,
     ExpressionBuiltinBytearray,
@@ -90,7 +99,7 @@ from nuitka.nodes.ConditionalNodes import (
     ExpressionConditional,
     StatementConditional
 )
-from nuitka.nodes.ConstantRefNodes import ExpressionConstantRef
+from nuitka.nodes.ConstantRefNodes import makeConstantRefNode
 from nuitka.nodes.ExecEvalNodes import (
     ExpressionBuiltinCompile,
     ExpressionBuiltinEval
@@ -123,13 +132,13 @@ from nuitka.nodes.VariableRefNodes import ExpressionVariableRef
 from nuitka.Options import isDebug, shallMakeModule
 from nuitka.PythonVersions import python_version
 from nuitka.tree.Helpers import (
+    makeSequenceCreationOrConstant,
     makeStatementsSequence,
     makeStatementsSequenceFromStatement
 )
 from nuitka.tree.ReformulationExecStatements import wrapEvalGlobalsAndLocals
 from nuitka.tree.ReformulationTryFinallyStatements import \
     makeTryFinallyStatement
-from nuitka.VariableRegistry import addVariableUsage
 
 from . import BuiltinOptimization
 
@@ -202,21 +211,29 @@ def import_extractor(node):
 
 def type_extractor(node):
     args = node.getCallArgs()
-    length = args.getIterationLength()
 
-    if length == 1:
+    if args is None:
+        iter_length = 0
+    else:
+        iter_length = args.getIterationLength()
+
+    if iter_length == 1:
         return BuiltinOptimization.extractBuiltinArgs(
             node          = node,
             builtin_class = ExpressionBuiltinType1,
             builtin_spec  = BuiltinOptimization.builtin_type1_spec
         )
-
-    else:
+    elif iter_length == 3:
         return BuiltinOptimization.extractBuiltinArgs(
             node          = node,
             builtin_class = ExpressionBuiltinType3,
             builtin_spec  = BuiltinOptimization.builtin_type3_spec
         )
+    else:
+        return makeRaiseExceptionReplacementExpressionFromInstance(
+            expression = node,
+            exception  = TypeError("type() takes 1 or 3 arguments")
+        )
 
 def iter_extractor(node):
     @calledWithBuiltinArgumentNamesDecorator
@@ -259,7 +276,39 @@ def next_extractor(node):
     return BuiltinOptimization.extractBuiltinArgs(
         node          = node,
         builtin_class = selectNextBuiltinClass,
-        builtin_spec  = BuiltinOptimization.builtin_iter_spec
+        builtin_spec  = BuiltinOptimization.builtin_next_spec
+    )
+
+
+def sum_extractor(node):
+    # Split up sumwith and without start value, one is much easier.
+    def selectSumBuiltinClass(sequence, start, source_ref):
+        if start is None:
+            return ExpressionBuiltinSum1(
+                sequence   = sequence,
+                source_ref = source_ref
+            )
+        else:
+            return ExpressionBuiltinSum2(
+                sequence   = sequence,
+                start      = start,
+                source_ref = source_ref
+            )
+
+    def makeSum0(source_ref):
+        # pylint: disable=W0613
+
+        return makeRaiseExceptionReplacementExpressionFromInstance(
+            expression = node,
+            exception  = TypeError("sum expected at least 1 arguments, got 0")
+        )
+
+
+    return BuiltinOptimization.extractBuiltinArgs(
+        node                = node,
+        builtin_class       = selectSumBuiltinClass,
+        builtin_spec        = BuiltinOptimization.builtin_sum_spec,
+        empty_special_class = makeSum0
     )
 
 
@@ -314,11 +363,19 @@ def chr_extractor(node):
     )
 
 def ord_extractor(node):
+    def makeOrd0(source_ref):
+        # pylint: disable=W0613
+
+        return makeRaiseExceptionReplacementExpressionFromInstance(
+            expression = node,
+            exception  = TypeError("ord() takes exactly one argument (0 given)")
+        )
+
     return BuiltinOptimization.extractBuiltinArgs(
         node                = node,
         builtin_class       = ExpressionBuiltinOrd,
         builtin_spec        = BuiltinOptimization.builtin_ord_spec,
-        empty_special_class = ExpressionBuiltinOrd0
+        empty_special_class = makeOrd0
     )
 
 def bin_extractor(node):
@@ -363,6 +420,16 @@ def repr_extractor(node):
         builtin_spec  = BuiltinOptimization.builtin_repr_spec
     )
 
+
+if python_version >= 300:
+    def ascii_extractor(node):
+        return BuiltinOptimization.extractBuiltinArgs(
+            node          = node,
+            builtin_class = ExpressionBuiltinAscii,
+            builtin_spec  = BuiltinOptimization.builtin_repr_spec
+        )
+
+
 def range_extractor(node):
     def selectRangeBuiltin(low, high, step, source_ref):
         if high is None:
@@ -384,22 +451,61 @@ def range_extractor(node):
                 source_ref = source_ref
             )
 
+    def makeRange0(source_ref):
+        # pylint: disable=W0613
+
+        return makeRaiseExceptionReplacementExpressionFromInstance(
+            expression = node,
+            exception  = TypeError("range expected at least 1 arguments, got 0")
+        )
 
     return BuiltinOptimization.extractBuiltinArgs(
         node                = node,
         builtin_class       = selectRangeBuiltin,
         builtin_spec        = BuiltinOptimization.builtin_range_spec,
-        empty_special_class = ExpressionBuiltinRange0
+        empty_special_class = makeRange0
     )
 
-if python_version < 300:
-    from nuitka.nodes.BuiltinRangeNodes import ExpressionBuiltinXrange
 
-    def xrange_extractor(node):
-        return BuiltinOptimization.extractBuiltinArgs(
-            node          = node,
-            builtin_class = ExpressionBuiltinXrange,
-            builtin_spec  = BuiltinOptimization.builtin_xrange_spec
+
+def xrange_extractor(node):
+    def selectXrangeBuiltin(low, high, step, source_ref):
+        if high is None:
+            return ExpressionBuiltinXrange1(
+                low        = low,
+                source_ref = source_ref
+            )
+        elif step is None:
+            return ExpressionBuiltinXrange2(
+                low        = low,
+                high       = high,
+                source_ref = source_ref
+            )
+        else:
+            return ExpressionBuiltinXrange3(
+                low        = low,
+                high       = high,
+                step       = step,
+                source_ref = source_ref
+            )
+
+    def makeXrange0(source_ref):
+        # pylint: disable=W0613
+
+        return makeRaiseExceptionReplacementExpressionFromInstance(
+            expression = node,
+            exception  = TypeError(
+                "xrange requires 1-3 int arguments"
+                    if python_version < 300 else
+                "range expected 1 arguments, got 0"
+            )
+        )
+
+    return BuiltinOptimization.extractBuiltinArgs(
+        node                = node,
+        builtin_class       = selectXrangeBuiltin,
+        builtin_spec        = BuiltinOptimization.builtin_xrange_spec,
+        empty_special_class = makeXrange0
     )
 
 
@@ -541,7 +647,7 @@ if python_version < 300:
                                 called     = ExpressionAttributeLookup(
                                     source         = ExpressionBuiltinOpen(
                                         filename   = filename,
-                                        mode       = ExpressionConstantRef(
+                                        mode       = makeConstantRefNode(
                                             constant   = "rU",
                                             source_ref = source_ref
                                         ),
@@ -628,7 +734,7 @@ def eval_extractor(node):
             )
         )
 
-        strip_choice =  ExpressionConstantRef(
+        strip_choice =  makeConstantRefNode(
             constant   = (" \t",),
             source_ref = source_ref
         )
@@ -649,7 +755,7 @@ def eval_extractor(node):
                     ),
                     source_ref = source_ref
                 ),
-                expression_yes = ExpressionConstantRef(
+                expression_yes = makeConstantRefNode(
                     constant   = (b" \t",),
                     source_ref = source_ref
                 ),
@@ -682,6 +788,21 @@ def eval_extractor(node):
             )
         ]
 
+        acceptable_builtin_types = [
+            ExpressionBuiltinAnonymousRef(
+                builtin_name = "code",
+                source_ref   = source_ref,
+            )
+        ]
+
+        if python_version >= 270:
+            acceptable_builtin_types.append(
+                ExpressionBuiltinRef(
+                    builtin_name = "memoryview",
+                    source_ref   = source_ref,
+                )
+            )
+
         statements = (
             StatementAssignmentVariable(
                 variable_ref = ExpressionTargetTempVariableRef(
@@ -698,9 +819,10 @@ def eval_extractor(node):
                             variable   = source_variable,
                             source_ref = source_ref
                         ),
-                        classes    = ExpressionBuiltinAnonymousRef(
-                            builtin_name = "code",
-                            source_ref   = source_ref,
+                        classes    = makeSequenceCreationOrConstant(
+                            sequence_kind = "tuple",
+                            elements      = acceptable_builtin_types,
+                            source_ref    = source_ref
                         ),
                         source_ref = source_ref
                     ),
@@ -838,12 +960,26 @@ def compile_extractor(node):
 
 
 def open_extractor(node):
+    def makeOpen0(source_ref):
+        # pylint: disable=W0613
+
+        return makeRaiseExceptionReplacementExpressionFromInstance(
+            expression = node,
+            exception  = TypeError(
+                "Required argument 'name' (pos 1) not found"
+                  if python_version < 300 else
+                "Required argument 'file' (pos 1) not found"
+            )
+        )
+
     return BuiltinOptimization.extractBuiltinArgs(
-        node          = node,
-        builtin_class = ExpressionBuiltinOpen,
-        builtin_spec  = BuiltinOptimization.builtin_open_spec
+        node                = node,
+        builtin_class       = ExpressionBuiltinOpen,
+        builtin_spec        = BuiltinOptimization.builtin_open_spec,
+        empty_special_class = makeOpen0
     )
 
+
 def super_extractor(node):
     @calledWithBuiltinArgumentNamesDecorator
     def wrapSuperBuiltin(type_arg, object_arg, source_ref):
@@ -870,8 +1006,6 @@ def super_extractor(node):
             not (type_arg_owner.isExpressionFunctionBody() or \
                  type_arg_owner.isExpressionClassBody()):
                 type_arg = None
-            else:
-                addVariableUsage(type_arg.getVariable(), provider)
 
             if type_arg is None:
                 return makeRaiseExceptionReplacementExpression(
@@ -883,7 +1017,8 @@ def super_extractor(node):
                 )
 
             if object_arg is None:
-                if provider.isExpressionGeneratorObjectBody():
+                if provider.isExpressionGeneratorObjectBody() or \
+                   provider.isExpressionCoroutineObjectBody():
                     parameter_provider = provider.getParentVariableProvider()
                 else:
                     parameter_provider = provider
@@ -993,6 +1128,22 @@ def hash_extractor(node):
         builtin_spec  = BuiltinOptimization.builtin_hash_spec
     )
 
+def format_extractor(node):
+    def makeFormat0(source_ref):
+        # pylint: disable=W0613
+
+        return makeRaiseExceptionReplacementExpressionFromInstance(
+            expression = node,
+            exception  = TypeError("format() takes at least 1 argument (0 given)")
+        )
+
+    return BuiltinOptimization.extractBuiltinArgs(
+        node                = node,
+        builtin_class       = ExpressionBuiltinFormat,
+        builtin_spec        = BuiltinOptimization.builtin_format_spec,
+        empty_special_class = makeFormat0
+    )
+
 
 _dispatch_dict = {
     "compile"    : compile_extractor,
@@ -1011,7 +1162,7 @@ _dispatch_dict = {
     "type"       : type_extractor,
     "iter"       : iter_extractor,
     "next"       : next_extractor,
-    "range"      : range_extractor,
+    "sum"        : sum_extractor,
     "tuple"      : tuple_extractor,
     "list"       : list_extractor,
     "dict"       : dict_extractor,
@@ -1031,18 +1182,26 @@ _dispatch_dict = {
     # TODO: Disabled for now, not handling all cases.
     # "bytearray"  : bytearray_extractor,
     "slice"      : slice_extractor,
-    "hash"       : hash_extractor
+    "hash"       : hash_extractor,
+    "format"     : format_extractor,
 }
 
 if python_version < 300:
+    # These are not in Python3
     _dispatch_dict["long"] = long_extractor
     _dispatch_dict["unicode"] = unicode_extractor
     _dispatch_dict["execfile"] = execfile_extractor
 
+    _dispatch_dict["xrange"] = xrange_extractor
+    _dispatch_dict["range"] = range_extractor
+
     # The handling of 'open' built-in for Python3 is not yet correct.
     _dispatch_dict["open"] = open_extractor
 else:
+    # The Python3 range is really an xrange, use that.
+    _dispatch_dict["range"] = xrange_extractor
     _dispatch_dict["exec"] = exec_extractor
+    _dispatch_dict["ascii"] = ascii_extractor
 
 def check():
     from nuitka.Builtins import builtin_names
