@@ -1,4 +1,4 @@
-#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -50,6 +50,7 @@ catching and passing in exceptions raised.
 """
 
 import sys
+from logging import warning
 
 from nuitka import Options, SourceCodeReferences, Tracing
 from nuitka.__past__ import long, unicode  # pylint: disable=W0622
@@ -61,18 +62,24 @@ from nuitka.nodes.AssignNodes import (
     StatementAssignmentVariable
 )
 from nuitka.nodes.AttributeNodes import ExpressionAttributeLookup
+from nuitka.nodes.BuiltinFormatNodes import (
+    ExpressionBuiltinAscii,
+    ExpressionBuiltinFormat
+)
+from nuitka.nodes.BuiltinTypeNodes import ExpressionBuiltinStr
 from nuitka.nodes.ConditionalNodes import (
     ExpressionConditional,
     StatementConditional
 )
-from nuitka.nodes.ConstantRefNodes import ExpressionConstantRef
+from nuitka.nodes.ConstantRefNodes import (
+    ExpressionConstantEllipsisRef,
+    ExpressionConstantNoneRef,
+    makeConstantRefNode
+)
 from nuitka.nodes.CoroutineNodes import ExpressionAsyncWait
 from nuitka.nodes.ExceptionNodes import StatementRaiseException
 from nuitka.nodes.GeneratorNodes import StatementGeneratorReturn
-from nuitka.nodes.ImportNodes import (
-    ExpressionImportModule,
-    ExpressionImportName
-)
+from nuitka.nodes.ImportNodes import ExpressionImportModule
 from nuitka.nodes.LoopNodes import StatementLoopBreak, StatementLoopContinue
 from nuitka.nodes.ModuleNodes import (
     CompiledPythonModule,
@@ -82,23 +89,27 @@ from nuitka.nodes.ModuleNodes import (
     PythonShlibModule
 )
 from nuitka.nodes.OperatorNodes import (
-    ExpressionOperationBinary,
-    ExpressionOperationUnary
+    ExpressionOperationUnary,
+    makeBinaryOperationNode
 )
 from nuitka.nodes.ReturnNodes import StatementReturn
 from nuitka.nodes.StatementNodes import StatementExpressionOnly
+from nuitka.nodes.StringConcatenationNodes import ExpressionStringConcatenation
 from nuitka.nodes.VariableRefNodes import ExpressionVariableRef
+from nuitka.Options import shallWarnUnusualCode
+from nuitka.plugins.Plugins import Plugins
 from nuitka.PythonVersions import python_version
 from nuitka.tree.ReformulationForLoopStatements import (
     buildAsyncForLoopNode,
     buildForLoopNode
 )
 from nuitka.tree.ReformulationWhileLoopStatements import buildWhileLoopNode
-from nuitka.utils import Utils
+from nuitka.utils import MemoryUsage, Utils
 
 from . import SyntaxErrors
 from .Helpers import (
     buildNode,
+    buildNodeList,
     buildStatementsNode,
     extractDocFromBody,
     getBuildContext,
@@ -106,7 +117,6 @@ from .Helpers import (
     makeModuleFrame,
     makeStatementsSequence,
     makeStatementsSequenceFromStatement,
-    makeStatementsSequenceOrStatement,
     mangleName,
     mergeStatements,
     parseSourceCodeToAst,
@@ -114,6 +124,7 @@ from .Helpers import (
 )
 from .ReformulationAssertStatements import buildAssertNode
 from .ReformulationAssignmentStatements import (
+    buildAnnAssignNode,
     buildAssignNode,
     buildDeleteNode,
     buildInplaceAssignNode
@@ -136,6 +147,7 @@ from .ReformulationFunctionStatements import (
 )
 from .ReformulationImportStatements import (
     buildImportFromNode,
+    buildImportModulesNode,
     checkFutureImportsOnlyAtStart
 )
 from .ReformulationLambdaExpressions import buildLambdaNode
@@ -150,7 +162,10 @@ from .ReformulationTryExceptStatements import buildTryExceptionNode
 from .ReformulationTryFinallyStatements import buildTryFinallyNode
 from .ReformulationWithStatements import buildAsyncWithNode, buildWithNode
 from .ReformulationYieldExpressions import buildYieldFromNode, buildYieldNode
-from .SourceReading import readSourceCodeFromFilename
+from .SourceReading import (
+    checkPythonVersionFromCode,
+    readSourceCodeFromFilename
+)
 from .VariableClosure import completeVariableClosures
 
 
@@ -163,7 +178,7 @@ def buildVariableReferenceNode(provider, node, source_ref):
 
 # Python3.4 or higher, True and False, are not given as variables anymore.
 def buildNamedConstantNode(node, source_ref):
-    return ExpressionConstantRef(
+    return makeConstantRefNode(
         constant   = node.value,
         source_ref = source_ref
     )
@@ -190,7 +205,7 @@ def buildConditionNode(provider, node, source_ref):
     )
 
 
-def _buildTryFinallyNode(provider, node, source_ref):
+def buildTryFinallyNode2(provider, node, source_ref):
     # Try/finally node statements of old style.
 
     return buildTryFinallyNode(
@@ -212,7 +227,7 @@ def buildTryNode(provider, node, source_ref):
 
     # Shortcut missing try/finally.
     if not node.handlers:
-        return _buildTryFinallyNode(provider, node, source_ref)
+        return buildTryFinallyNode2(provider, node, source_ref)
 
     if not node.finalbody:
         return buildTryExceptionNode(
@@ -284,86 +299,17 @@ def buildRaiseNode(provider, node, source_ref):
 
     return result
 
-def buildImportModulesNode(provider, node, source_ref):
-    # Import modules statement. As described in the developer manual, these
-    # statements can be treated as several ones.
-
-    import_names   = [
-        ( import_desc.name, import_desc.asname )
-        for import_desc in
-        node.names
-    ]
-
-    import_nodes = []
-
-    for import_desc in import_names:
-        module_name, local_name = import_desc
-
-        module_topname = module_name.split('.')[0]
-
-        # Note: The "level" of import is influenced by the future absolute
-        # imports.
-        level = 0 if source_ref.getFutureSpec().isAbsoluteImport() else -1
-
-        if local_name:
-            # If is gets a local name, the real name must be used as a
-            # temporary value only, being looked up recursively.
-
-            import_node = ExpressionImportModule(
-                module_name = module_name,
-                import_list = None,
-                level       = level,
-                source_ref  = source_ref
-            )
-
-            for import_name in module_name.split('.')[1:]:
-                import_node = ExpressionImportName(
-                    module      = import_node,
-                    import_name = import_name,
-                    source_ref  = source_ref
-                )
-        else:
-            import_node = ExpressionImportModule(
-                module_name = module_name,
-                import_list = None,
-                level       = level,
-                source_ref  = source_ref
-            )
-
-        # If a name was given, use the one provided, otherwise the import gives
-        # the top level package name given for assignment of the imported
-        # module.
-
-        import_nodes.append(
-            StatementAssignmentVariable(
-                variable_ref = ExpressionTargetVariableRef(
-                    variable_name = mangleName(
-                        local_name
-                          if local_name is not None else
-                        module_topname,
-                        provider
-                    ),
-                    source_ref    = source_ref
-                ),
-                source       = import_node,
-                source_ref   = source_ref
-            )
-        )
-
-    # Note: Each import is sequential. It will potentially succeed, and the
-    # failure of a later one is not changing that one bit . We can therefore
-    # have a sequence of imports that only import one thing therefore.
-    return makeStatementsSequenceOrStatement(
-        statements = import_nodes,
-        source_ref = source_ref
-    )
-
 
 def handleGlobalDeclarationNode(provider, node, source_ref):
 
-    # On the module level, there is nothing to do. TODO: Probably a warning
-    # would be warranted.
+    # On the module level, there is nothing to do.
     if provider.isCompiledPythonModule():
+        if shallWarnUnusualCode():
+            warning(
+                "%s: Using 'global' statement on module level has no effect.",
+                source_ref.getAsString(),
+            )
+
         return None
 
     # Need to catch the error of declaring a parameter variable as global
@@ -374,18 +320,13 @@ def handleGlobalDeclarationNode(provider, node, source_ref):
         for variable_name in node.names:
             if variable_name in parameters.getParameterNames():
                 SyntaxErrors.raiseSyntaxError(
-                    reason     = "name '%s' is %s and global" % (
+                    "name '%s' is %s and global" % (
                         variable_name,
                         "local"
                           if python_version < 300 else
                         "parameter"
                     ),
-                    source_ref = (
-                        source_ref
-                          if not Options.isFullCompat() or \
-                             python_version >= 340 else
-                        provider.getSourceReference()
-                    )
+                    source_ref.atColumnNumber(node.col_offset)
                 )
 
     # The module the "global" statement refers to.
@@ -421,8 +362,8 @@ def handleGlobalDeclarationNode(provider, node, source_ref):
            provider.isExpressionClassBody() and \
            closure_variable.getName() == "__class__":
             SyntaxErrors.raiseSyntaxError(
-                reason     = "cannot make __class__ global",
-                source_ref = source_ref
+                "cannot make __class__ global",
+                source_ref
             )
 
         provider.registerProvidedVariable(
@@ -449,20 +390,13 @@ def handleNonlocalDeclarationNode(provider, node, source_ref):
     for variable_name in node.names:
         if variable_name in parameter_names:
             SyntaxErrors.raiseSyntaxError(
-                reason       = "name '%s' is parameter and nonlocal" % (
+                "name '%s' is parameter and nonlocal" % (
                     variable_name
                 ),
-                source_ref   = None
-                                 if Options.isFullCompat() and \
-                                 python_version < 340 else
-                               source_ref,
-                display_file = not Options.isFullCompat() or \
-                               python_version >= 340,
-                display_line = not Options.isFullCompat() or \
-                               python_version >= 340
+                source_ref.atColumnNumber(node.col_offset)
             )
 
-    provider.addNonlocalsDeclaration(node.names, source_ref)
+    provider.addNonlocalsDeclaration(node.names, source_ref.atColumnNumber(node.col_offset))
 
     return None
 
@@ -470,7 +404,7 @@ def handleNonlocalDeclarationNode(provider, node, source_ref):
 def buildStringNode(node, source_ref):
     assert type(node.s) in (str, unicode)
 
-    return ExpressionConstantRef(
+    return makeConstantRefNode(
         constant      = node.s,
         source_ref    = source_ref,
         user_provided = True
@@ -480,7 +414,7 @@ def buildStringNode(node, source_ref):
 def buildNumberNode(node, source_ref):
     assert type(node.n) in (int, long, float, complex), type(node.n)
 
-    return ExpressionConstantRef(
+    return makeConstantRefNode(
         constant      = node.n,
         source_ref    = source_ref,
         user_provided = True
@@ -488,7 +422,7 @@ def buildNumberNode(node, source_ref):
 
 
 def buildBytesNode(node, source_ref):
-    return ExpressionConstantRef(
+    return makeConstantRefNode(
         constant      = node.s,
         source_ref    = source_ref,
         user_provided = True
@@ -496,8 +430,7 @@ def buildBytesNode(node, source_ref):
 
 
 def buildEllipsisNode(source_ref):
-    return ExpressionConstantRef(
-        constant      = Ellipsis,
+    return ExpressionConstantEllipsisRef(
         source_ref    = source_ref,
         user_provided = True
     )
@@ -506,22 +439,12 @@ def buildEllipsisNode(source_ref):
 def buildStatementLoopContinue(node, source_ref):
     # Python forbids this, although technically it's probably not much of
     # an issue.
+    source_ref = source_ref.atColumnNumber(node.col_offset)
+
     if getBuildContext() == "finally":
-        if not Options.isFullCompat() or python_version >= 300:
-            col_offset = node.col_offset - 9
-        else:
-            col_offset = None
-
-        if python_version >= 300 and Options.isFullCompat():
-            source_line = ""
-        else:
-            source_line = None
-
         SyntaxErrors.raiseSyntaxError(
             "'continue' not supported inside 'finally' clause",
-            source_ref,
-            col_offset  = col_offset,
-            source_line = source_line
+            source_ref
         )
 
     return StatementLoopContinue(
@@ -534,7 +457,7 @@ def buildStatementLoopBreak(provider, node, source_ref):
     # pylint: disable=W0613
 
     return StatementLoopBreak(
-        source_ref = source_ref
+        source_ref = source_ref.atColumnNumber(node.col_offset)
     )
 
 
@@ -550,12 +473,7 @@ def buildReturnNode(provider, node, source_ref):
     if provider.isExpressionClassBody() or provider.isCompiledPythonModule():
         SyntaxErrors.raiseSyntaxError(
             "'return' outside function",
-            source_ref,
-            None if python_version < 300 else (
-                node.col_offset
-                  if provider.isCompiledPythonModule() else
-                node.col_offset+4
-            )
+            source_ref.atColumnNumber(node.col_offset)
         )
 
     expression = buildNode(provider, node.value, source_ref, allow_none = True)
@@ -564,12 +482,11 @@ def buildReturnNode(provider, node, source_ref):
         if expression is not None and python_version < 330:
             SyntaxErrors.raiseSyntaxError(
                 "'return' with argument inside generator",
-                source_ref = source_ref,
+                source_ref.atColumnNumber(node.col_offset)
             )
 
     if expression is None:
-        expression = ExpressionConstantRef(
-            constant      = None,
+        expression = ExpressionConstantNoneRef(
             source_ref    = source_ref,
             user_provided = True
         )
@@ -623,7 +540,7 @@ def buildBinaryOpNode(provider, node, source_ref):
     left       = buildNode(provider, node.left, source_ref)
     right      = buildNode(provider, node.right, source_ref)
 
-    result = ExpressionOperationBinary(
+    result = makeBinaryOperationNode(
         operator   = operator,
         left       = left,
         right      = right,
@@ -661,10 +578,54 @@ def buildAwaitNode(provider, node, source_ref):
     )
 
 
+def buildFormattedValueNode(provider, node, source_ref):
+    value = buildNode(provider, node.value, source_ref)
+
+    conversion = node.conversion % 4 if node.conversion > 0 else 0
+
+    if conversion == 0:
+        pass
+    elif conversion == 3:
+        value = ExpressionBuiltinStr(
+            value      = value,
+            encoding   = None,
+            errors     = None,
+            source_ref = source_ref
+        )
+    elif conversion == 2:
+        value = ExpressionOperationUnary(
+            operator   = "Repr",
+            operand    = value,
+            source_ref = source_ref
+        )
+    elif conversion == 1:
+        value = ExpressionBuiltinAscii(
+            value      = value,
+            source_ref = source_ref
+        )
+    else:
+        assert False, conversion
+
+    return ExpressionBuiltinFormat(
+        value       = value,
+        format_spec = buildNode(provider, node.format_spec, source_ref, allow_none = True),
+        source_ref  = source_ref
+    )
+
+
+def buildJoinedStrNode(provider, node, source_ref):
+    return ExpressionStringConcatenation(
+        values     = buildNodeList(provider, node.values, source_ref),
+        source_ref = source_ref
+    )
+
+
+
 setBuildingDispatchers(
     path_args3 = {
         "Name"              : buildVariableReferenceNode,
         "Assign"            : buildAssignNode,
+        "AnnAssign"         : buildAnnAssignNode,
         "Delete"            : buildDeleteNode,
         "Lambda"            : buildLambdaNode,
         "GeneratorExp"      : buildGeneratorExpressionNode,
@@ -683,7 +644,7 @@ setBuildingDispatchers(
         "Global"            : handleGlobalDeclarationNode,
         "Nonlocal"          : handleNonlocalDeclarationNode,
         "TryExcept"         : buildTryExceptionNode,
-        "TryFinally"        : _buildTryFinallyNode,
+        "TryFinally"        : buildTryFinallyNode2,
         "Try"               : buildTryNode,
         "Raise"             : buildRaiseNode,
         "Import"            : buildImportModulesNode,
@@ -711,6 +672,8 @@ setBuildingDispatchers(
         "AugAssign"         : buildInplaceAssignNode,
         "IfExp"             : buildConditionalExpressionNode,
         "Break"             : buildStatementLoopBreak,
+        "JoinedStr"         : buildJoinedStrNode,
+        "FormattedValue"    : buildFormattedValueNode,
     },
     path_args2 = {
         "NameConstant" : buildNamedConstantNode,
@@ -783,7 +746,7 @@ def buildParseTree(provider, source_code, source_ref, is_module, is_main):
                     variable_name = "__doc__",
                     source_ref    = internal_source_ref
                 ),
-                source       = ExpressionConstantRef(
+                source       = makeConstantRefNode(
                     constant      = doc,
                     source_ref    = internal_source_ref,
                     user_provided = True
@@ -818,8 +781,7 @@ def buildParseTree(provider, source_code, source_ref, is_module, is_main):
                     variable_name = "__cached__",
                     source_ref    = internal_source_ref
                 ),
-                source       = ExpressionConstantRef(
-                    constant      = None,
+                source       = ExpressionConstantNoneRef(
                     source_ref    = internal_source_ref,
                     user_provided = True
                 ),
@@ -836,7 +798,7 @@ def buildParseTree(provider, source_code, source_ref, is_module, is_main):
                     variable_name = "__package__",
                     source_ref    = internal_source_ref
                 ),
-                source       = ExpressionConstantRef(
+                source       = makeConstantRefNode(
                     constant      = provider.getFullName()
                                       if provider.isCompiledPythonPackage() else
                                     provider.getPackage(),
@@ -858,7 +820,7 @@ def buildParseTree(provider, source_code, source_ref, is_module, is_main):
                     variable_name = "__initializing__",
                     source_ref    = internal_source_ref
                 ),
-                source       = ExpressionConstantRef(
+                source       = makeConstantRefNode(
                     constant      = True,
                     source_ref    = internal_source_ref,
                     user_provided = True
@@ -867,6 +829,24 @@ def buildParseTree(provider, source_code, source_ref, is_module, is_main):
             )
         )
 
+    if python_version >= 360:
+        # Set "__annotations__" on module level to {}
+        statements.append(
+            StatementAssignmentVariable(
+                variable_ref = ExpressionTargetVariableRef(
+                    variable_name = "__annotations__",
+                    source_ref    = internal_source_ref
+                ),
+                source       = makeConstantRefNode(
+                    constant      = {},
+                    source_ref    = internal_source_ref,
+                    user_provided = True
+                ),
+                source_ref   = internal_source_ref
+            )
+        )
+
+
     # Now the module body if there is any at all.
     if result is not None:
         statements.extend(
@@ -881,7 +861,7 @@ def buildParseTree(provider, source_code, source_ref, is_module, is_main):
                     variable_name = "__initializing__",
                     source_ref    = internal_source_ref
                 ),
-                source       = ExpressionConstantRef(
+                source       = makeConstantRefNode(
                     constant      = False,
                     source_ref    = internal_source_ref,
                     user_provided = True
@@ -902,7 +882,7 @@ def buildParseTree(provider, source_code, source_ref, is_module, is_main):
 
 
 def decideModuleTree(filename, package, is_shlib, is_top, is_main):
-    # Many variables, branches, due to the many cases, pylint: disable=R0912
+    # Many variables, branches, due to the many cases, pylint: disable=R0912,R0915
 
     assert package is None or type(package) is str
     assert filename is not None
@@ -961,14 +941,22 @@ def decideModuleTree(filename, package, is_shlib, is_top, is_main):
         elif is_main:
             result = PythonMainModule(
                 main_added = main_added,
+                mode       = Plugins.decideCompilation("__main__", source_ref),
                 source_ref = source_ref
             )
         else:
+            if package is not None:
+                full_name = package + '.' + module_name
+            else:
+                full_name = module_name
+
             result = CompiledPythonModule(
                 name         = module_name,
                 package_name = package,
+                mode         = Plugins.decideCompilation(full_name, source_ref),
                 source_ref   = source_ref
             )
+
     elif Importing.isPackageDir(filename):
         if is_top:
             package_name = Utils.splitpath(filename)[-1]
@@ -988,9 +976,15 @@ def decideModuleTree(filename, package, is_shlib, is_top, is_main):
                 filename = Utils.abspath(source_filename),
             )
 
+            if package is not None:
+                full_name = package + '.' + package_name
+            else:
+                full_name = package_name
+
             result = CompiledPythonPackage(
                 name         = package_name,
                 package_name = package,
+                mode         = Plugins.decideCompilation(full_name, source_ref),
                 source_ref   = source_ref
             )
     else:
@@ -1021,8 +1015,8 @@ class CodeTooComplexCode(Exception):
 
 
 def createModuleTree(module, source_ref, source_code, is_main):
-    if Options.isShowProgress():
-        memory_watch = Utils.MemoryWatch()
+    if Options.isShowMemory():
+        memory_watch = MemoryUsage.MemoryWatch()
 
     try:
         module_body = buildParseTree(
@@ -1050,7 +1044,7 @@ def createModuleTree(module, source_ref, source_code, is_main):
 
     completeVariableClosures(module)
 
-    if Options.isShowProgress():
+    if Options.isShowMemory():
         memory_watch.finish()
 
         Tracing.printLine(
@@ -1073,11 +1067,16 @@ def buildModuleTree(filename, package, is_top, is_main):
     # If there is source code associated (not the case for namespace packages of
     # Python3.3 or higher, then read it.
     if source_filename is not None:
+        source_code = readSourceCodeFromFilename(module.getFullName(), source_filename)
+
+        if is_main:
+            checkPythonVersionFromCode(source_code)
+
         # Read source code.
         createModuleTree(
             module      = module,
             source_ref  = source_ref,
-            source_code = readSourceCodeFromFilename(module.getFullName(), source_filename),
+            source_code = source_code,
             is_main     = is_main
         )
 
