@@ -1,4 +1,4 @@
-#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -39,14 +39,14 @@ from nuitka.nodes.ClassNodes import (
     ExpressionSelectMetaclass
 )
 from nuitka.nodes.CodeObjectSpecs import CodeObjectSpec
-from nuitka.nodes.ComparisonNodes import ExpressionComparison
+from nuitka.nodes.ComparisonNodes import ExpressionComparisonIn
 from nuitka.nodes.ConditionalNodes import (
     ExpressionConditional,
     StatementConditional
 )
-from nuitka.nodes.ConstantRefNodes import ExpressionConstantRef
+from nuitka.nodes.ConstantRefNodes import makeConstantRefNode
 from nuitka.nodes.ContainerMakingNodes import ExpressionMakeTuple
-from nuitka.nodes.ContainerOperationNodes import (
+from nuitka.nodes.DictionaryNodes import (
     ExpressionDictOperationGet,
     StatementDictOperationRemove
 )
@@ -70,9 +70,9 @@ from nuitka.nodes.VariableRefNodes import (
 from nuitka.PythonVersions import python_version
 
 from .Helpers import (
+    buildFrameNode,
     buildNode,
     buildNodeList,
-    buildStatementsNode,
     extractDocFromBody,
     getKind,
     makeDictCreationOrConstant,
@@ -85,7 +85,7 @@ from .ReformulationTryFinallyStatements import makeTryFinallyStatement
 
 def _buildClassNode3(provider, node, source_ref):
     # Many variables, due to the huge re-formulation that is going on here,
-    # which just has the complexity, pylint: disable=R0914
+    # which just has the complexity, pylint: disable=R0914,R0915
 
     # This function is the Python3 special case with special re-formulation as
     # according to developer manual.
@@ -153,15 +153,16 @@ def _buildClassNode3(provider, node, source_ref):
         )
 
     code_object = CodeObjectSpec(
-        code_name     = node.name,
-        code_kind     = "Class",
-        arg_names     = (),
-        kw_only_count = 0,
-        has_starlist  = False,
-        has_stardict  = False
+        co_name           = node.name,
+        co_kind           = "Class",
+        co_varnames       = (),
+        co_argcount       = 0,
+        co_kwonlyargcount = 0,
+        co_has_starlist   = False,
+        co_has_stardict   = False
     )
 
-    body = buildStatementsNode(
+    body = buildFrameNode(
         provider    = class_creation_function,
         nodes       = class_statement_nodes,
         code_object = code_object,
@@ -192,7 +193,7 @@ def _buildClassNode3(provider, node, source_ref):
                 variable      = module_variable,
                 source_ref    = source_ref
             ),
-            source       = ExpressionConstantRef(
+            source       = makeConstantRefNode(
                 constant      = provider.getParentModule().getFullName(),
                 source_ref    = source_ref,
                 user_provided = True
@@ -213,7 +214,7 @@ def _buildClassNode3(provider, node, source_ref):
                     variable      = doc_variable,
                     source_ref    = source_ref
                 ),
-                source       = ExpressionConstantRef(
+                source       = makeConstantRefNode(
                     constant      = class_doc,
                     source_ref    = source_ref,
                     user_provided = True
@@ -230,7 +231,7 @@ def _buildClassNode3(provider, node, source_ref):
         )
 
         if python_version < 340:
-            qualname_ref = ExpressionConstantRef(
+            qualname_ref = makeConstantRefNode(
                 constant      = qualname,
                 source_ref    = source_ref,
                 user_provided = True
@@ -256,8 +257,31 @@ def _buildClassNode3(provider, node, source_ref):
         if python_version >= 340:
             qualname_assign = statements[-1]
 
+    if python_version >= 360 and \
+       class_creation_function.needsAnnotationsDictionary():
+        annotations_variable = class_creation_function.getVariableForAssignment(
+            "__annotations__"
+        )
+
+        statements.append(
+            StatementAssignmentVariable(
+                variable_ref = ExpressionTargetVariableRef(
+                    variable_name = "__annotations__",
+                    variable      = annotations_variable,
+                    source_ref    = source_ref
+                ),
+                source       = makeConstantRefNode(
+                    constant      = {},
+                    source_ref    = source_ref,
+                    user_provided = True
+                ),
+                source_ref   = source_ref
+            )
+        )
+
+    statements.append(body)
+
     statements += [
-        body,
         StatementAssignmentVariable(
             variable_ref = class_target_variable_ref,
             source       = ExpressionCall(
@@ -268,7 +292,7 @@ def _buildClassNode3(provider, node, source_ref):
                 args       = makeSequenceCreationOrConstant(
                     sequence_kind = "tuple",
                     elements      = (
-                        ExpressionConstantRef(
+                        makeConstantRefNode(
                             constant      = node.name,
                             source_ref    = source_ref,
                             user_provided = True
@@ -371,7 +395,7 @@ def _buildClassNode3(provider, node, source_ref):
             ),
             source       = makeDictCreationOrConstant(
                 keys       = [
-                    ExpressionConstantRef(
+                    makeConstantRefNode(
                         constant      = keyword.arg,
                         source_ref    = source_ref,
                         user_provided = True
@@ -395,9 +419,8 @@ def _buildClassNode3(provider, node, source_ref):
             ),
             source       = ExpressionSelectMetaclass(
                 metaclass  = ExpressionConditional(
-                    condition      = ExpressionComparison(
-                        comparator = "In",
-                        left       = ExpressionConstantRef(
+                    condition      = ExpressionComparisonIn(
+                        left       = makeConstantRefNode(
                             constant      = "metaclass",
                             source_ref    = source_ref,
                             user_provided = True
@@ -413,7 +436,7 @@ def _buildClassNode3(provider, node, source_ref):
                             variable   = tmp_class_decl_dict,
                             source_ref = source_ref
                         ),
-                        key        = ExpressionConstantRef(
+                        key        = makeConstantRefNode(
                             constant      = "metaclass",
                             source_ref    = source_ref,
                             user_provided = True
@@ -435,7 +458,7 @@ def _buildClassNode3(provider, node, source_ref):
                                     variable   = tmp_bases,
                                     source_ref = source_ref
                                 ),
-                                subscript  = ExpressionConstantRef(
+                                subscript  = makeConstantRefNode(
                                     constant      = 0,
                                     source_ref    = source_ref,
                                     user_provided = True
@@ -457,9 +480,8 @@ def _buildClassNode3(provider, node, source_ref):
             source_ref   = source_ref_orig
         ),
         StatementConditional(
-            condition  = ExpressionComparison(
-                comparator = "In",
-                left       = ExpressionConstantRef(
+            condition  = ExpressionComparisonIn(
+                left       = makeConstantRefNode(
                     constant      = "metaclass",
                     source_ref    = source_ref,
                     user_provided = True
@@ -477,7 +499,7 @@ def _buildClassNode3(provider, node, source_ref):
                         variable   = tmp_class_decl_dict,
                         source_ref = source_ref
                     ),
-                    key        = ExpressionConstantRef(
+                    key        = makeConstantRefNode(
                         constant      = "metaclass",
                         source_ref    = source_ref,
                         user_provided = True
@@ -498,14 +520,14 @@ def _buildClassNode3(provider, node, source_ref):
                         variable   = tmp_metaclass,
                         source_ref = source_ref
                     ),
-                    name       = ExpressionConstantRef(
+                    name       = makeConstantRefNode(
                         constant      = "__prepare__",
                         source_ref    = source_ref,
                         user_provided = True
                     ),
                     source_ref = source_ref
                 ),
-                expression_no  = ExpressionConstantRef(
+                expression_no  = makeConstantRefNode(
                     constant      = {},
                     source_ref    = source_ref,
                     user_provided = True
@@ -521,7 +543,7 @@ def _buildClassNode3(provider, node, source_ref):
                     ),
                     args       = ExpressionMakeTuple(
                         elements   = (
-                            ExpressionConstantRef(
+                            makeConstantRefNode(
                                 constant      = node.name,
                                 source_ref    = source_ref,
                                 user_provided = True
@@ -599,15 +621,16 @@ def _buildClassNode2(provider, node, source_ref):
     )
 
     code_object = CodeObjectSpec(
-        code_name     = node.name,
-        code_kind     = "Class",
-        arg_names     = (),
-        kw_only_count = 0,
-        has_starlist  = False,
-        has_stardict  = False
+        co_name           = node.name,
+        co_kind           = "Class",
+        co_varnames       = (),
+        co_argcount       = 0,
+        co_kwonlyargcount = 0,
+        co_has_starlist   = False,
+        co_has_stardict   = False
     )
 
-    body = buildStatementsNode(
+    body = buildFrameNode(
         provider    = function_body,
         nodes       = class_statement_nodes,
         code_object = code_object,
@@ -626,7 +649,7 @@ def _buildClassNode2(provider, node, source_ref):
                 variable_name = "__module__",
                 source_ref    = source_ref
             ),
-            source       = ExpressionConstantRef(
+            source       = makeConstantRefNode(
                 constant      = provider.getParentModule().getFullName(),
                 source_ref    = source_ref,
                 user_provided = True
@@ -642,7 +665,7 @@ def _buildClassNode2(provider, node, source_ref):
                     variable_name = "__doc__",
                     source_ref    = source_ref
                 ),
-                source       = ExpressionConstantRef(
+                source       = makeConstantRefNode(
                     constant      = class_doc,
                     source_ref    = source_ref,
                     user_provided = True
@@ -722,9 +745,8 @@ def _buildClassNode2(provider, node, source_ref):
                 source_ref = source_ref
             ),
             source       = ExpressionConditional(
-                condition      =  ExpressionComparison(
-                    comparator = "In",
-                    left       = ExpressionConstantRef(
+                condition      =  ExpressionComparisonIn(
+                    left       = makeConstantRefNode(
                         constant      = "__metaclass__",
                         source_ref    = source_ref,
                         user_provided = True
@@ -740,7 +762,7 @@ def _buildClassNode2(provider, node, source_ref):
                         variable   = tmp_class_dict,
                         source_ref = source_ref
                     ),
-                    key        = ExpressionConstantRef(
+                    key        = makeConstantRefNode(
                         constant      = "__metaclass__",
                         source_ref    = source_ref,
                         user_provided = True
@@ -771,7 +793,7 @@ def _buildClassNode2(provider, node, source_ref):
                 ),
                 args       = ExpressionMakeTuple(
                     elements   = (
-                        ExpressionConstantRef(
+                        makeConstantRefNode(
                             constant      = node.name,
                             source_ref    = source_ref,
                             user_provided = True
