@@ -1,4 +1,4 @@
-#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -22,7 +22,6 @@ source code comments with developer manual sections.
 
 """
 
-from nuitka import Options
 from nuitka.nodes.AssignNodes import (
     ExpressionTargetTempVariableRef,
     ExpressionTempVariableRef,
@@ -35,7 +34,7 @@ from nuitka.nodes.ComparisonNodes import (
     ExpressionComparisonIs
 )
 from nuitka.nodes.ConditionalNodes import StatementConditional
-from nuitka.nodes.ConstantRefNodes import ExpressionConstantRef
+from nuitka.nodes.ConstantRefNodes import makeConstantRefNode
 from nuitka.nodes.ExceptionNodes import (
     ExpressionCaughtExceptionTypeRef,
     ExpressionCaughtExceptionValueRef
@@ -87,7 +86,7 @@ def makeTryExceptNoRaise(provider, temp_scope, tried, handling, no_raise,
                     variable   = tmp_handler_indicator_variable,
                     source_ref = source_ref.atInternal()
                 ),
-                source       = ExpressionConstantRef(
+                source       = makeConstantRefNode(
                     constant   = False,
                     source_ref = source_ref
                 ),
@@ -118,7 +117,7 @@ def makeTryExceptNoRaise(provider, temp_scope, tried, handling, no_raise,
                     variable   = tmp_handler_indicator_variable,
                     source_ref = source_ref
                 ),
-                right      = ExpressionConstantRef(
+                right      = makeConstantRefNode(
                     constant   = True,
                     source_ref = source_ref
                 ),
@@ -141,7 +140,7 @@ def makeTryExceptNoRaise(provider, temp_scope, tried, handling, no_raise,
                 variable   = tmp_handler_indicator_variable,
                 source_ref = source_ref.atInternal()
             ),
-            source       = ExpressionConstantRef(
+            source       = makeConstantRefNode(
                 constant   = True,
                 source_ref = source_ref
             ),
@@ -336,6 +335,7 @@ def buildTryExceptionNode(provider, node, source_ref):
                         source_ref = source_ref
                     ),
                     final      = buildDeleteStatementFromDecoded(
+                        node       = node, # Won't be used.
                         kind       = kind,
                         detail     = detail,
                         source_ref = source_ref
@@ -361,11 +361,9 @@ def buildTryExceptionNode(provider, node, source_ref):
         if exception_types is None:
             if handler is not node.handlers[-1]:
                 SyntaxErrors.raiseSyntaxError(
-                    reason     = "default 'except:' must be last",
-                    source_ref = source_ref.atLineNumber(
-                        handler.lineno-1
-                          if Options.isFullCompat() else
-                        handler.lineno
+                    "default 'except:' must be last",
+                    source_ref.atLineNumber(handler.lineno).atColumnNumber(
+                        handler.col_offset
                     )
                 )
 
