@@ -1,4 +1,4 @@
-#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python tests originally created or extracted from other peoples work. The
 #     parts were too small to be protected.
@@ -883,6 +883,22 @@ def simpleFunction105():
 
     del g
 
+def simpleFunction106():
+    # Call a PyCFunction with a single argument.
+    return sys.getsizeof(type)
+
+def simpleFunction107():
+    # Call a PyCFunction with a single argument.
+    return sum(i for i in range(x))
+
+def simpleFunction108():
+    # Call a PyCFunction with a single argument.
+    return sum((i for i in range(x)), 17)
+
+def simpleFunction109():
+    # Call a PyCFunction that looks like a method call.
+    sys.exc_info()
+
 ####################################
 
 
