@@ -1,5 +1,5 @@
 #!/usr/bin/env python
-#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python tests originally created or extracted from other peoples work. The
 #     parts were too small to be protected.
@@ -51,6 +51,7 @@ tmp_dir = getTempDir()
 PACKAGE_LIST = (
     "nuitka",
     "nuitka/nodes",
+    "nuitka/nodes/shapes",
     "nuitka/tree",
     "nuitka/importing",
     "nuitka/build",
@@ -196,6 +197,7 @@ def executePASS1():
         os.environ["PYTHON"],
         nuitka_main_path,
         "--recurse-none",
+        "--plugin-enable=pylint-warnings",
         "--output-dir=.",
         "--python-flag=-S",
         "nuitka.py"
@@ -373,8 +375,9 @@ def executePASS5():
         "--plugin-enable=pylint-warnings",
         "--output-dir=%s" % tmp_dir,
         "--recurse-all",
-        "--recurse-not-to=PyQt4",
+        "--recurse-not-to=PyQt5",
         "--recurse-not-to=nuitka.build.inline_copy",
+        "--recurse-not-to=nuitka.build.include",
         "--recurse-dir=%s" % path,
         "--module",
         path
