@@ -1,4 +1,4 @@
-#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -28,7 +28,7 @@ def my_print(*args, **kwargs):
 
     sys.stdout.flush()
 
-
+# TODO: Use nuitka.utils.Execution instead.
 def check_output(*popenargs, **kwargs):
     if "stdout" in kwargs:
         raise ValueError("stdout argument not allowed, it will be overridden.")
@@ -178,7 +178,12 @@ def convertUsing2to3(path, force = False):
     filename = os.path.basename(path)
 
     new_path = os.path.join(getTempDir(), filename)
-    shutil.copy(path, new_path)
+
+    # This may already be a temp file, e.g. because of construct creation.
+    try:
+        shutil.copy(path, new_path)
+    except shutil.SameFileError:  # @UndefinedVariable
+        pass
 
     # On Windows, we cannot rely on 2to3 to be in the path.
     if os.name == "nt":
@@ -311,6 +316,14 @@ def hasDebugPython():
     if os.path.exists(debug_python):
         return True
 
+    # On Windows systems, these work.
+    debug_python = os.environ["PYTHON"]
+    if debug_python.lower().endswith(".exe"):
+        debug_python = debug_python[:-4]
+    debug_python = debug_python + "_d.exe"
+    if os.path.exists(debug_python):
+        return True
+
     # For other Python, if it's the one also executing the runner, which is
     # very probably the case, we check that. We don't check the provided
     # binary here, this could be done as well.
@@ -705,7 +718,10 @@ def createSearchMode():
         return SearchModeImmediate()
 
 def reportSkip(reason, dirname, filename):
-    my_print("Skipped, %s (%s)." % (os.path.join(dirname, filename), reason))
+    case = os.path.join(dirname, filename)
+    case = os.path.normpath(case)
+
+    my_print("Skipped, %s (%s)." % (case, reason))
 
 def executeReferenceChecked(prefix, names, tests_skipped, tests_stderr):
     import gc
